//
// Created by Артем Мурадян on 18.01.2025.
//
#include "matrix.h"
#include <gtest/gtest.h>
#include <vector>

// Тест: Конструктор по умолчанию
TEST(MatrixTests, DefaultConstructor) {
    linalg::Matrix<int> m;
    EXPECT_EQ(m.rows(), 0);
    EXPECT_EQ(m.columns(), 0);
    EXPECT_EQ(m.size(), 0);
    EXPECT_EQ(m.capacity(), 0);
    EXPECT_EQ(m.begin(), nullptr); // Указатель должен быть пустым.
}

// Тест: Инициализация размером
TEST(MatrixTests, SizeConstructor) {
    linalg::Matrix<double> m(3, 4);
    EXPECT_EQ(m.rows(), 3);
    EXPECT_EQ(m.columns(), 4);
    EXPECT_EQ(m.size(), 12);
    EXPECT_GE(m.capacity(), 12); // Ёмкость может быть больше или равна.
}

// Тест: Копирование матрицы
TEST(MatrixTests, CopyConstructor) {
    linalg::Matrix<int> m1 = {{1, 2}, {3, 4}};
    const linalg::Matrix<int>& m2(m1);
    EXPECT_EQ(m2.rows(), 2);
    EXPECT_EQ(m2.columns(), 2);
    EXPECT_EQ(m2(0, 0), 1);
    EXPECT_EQ(m2(1, 1), 4);
}

// Тест: Присваивание матриц
TEST(MatrixTests, AssignmentOperator) {
    linalg::Matrix<int> m1 = {{1, 2}, {3, 4}};
    linalg::Matrix<int> m2;
    m2 = m1;

    EXPECT_EQ(m2.rows(), 2);
    EXPECT_EQ(m2.columns(), 2);
    EXPECT_EQ(m2(0, 0), 1);
    EXPECT_EQ(m2(1, 1), 4);
}

// Тест: Присваивание с преобразованием типов
TEST(MatrixTests, TypeConversionAssignment) {
    linalg::Matrix<int> m1 = {{1, 2}, {3, 4}};
    linalg::Matrix<double> m2;
    m2 = m1; // Присваивание с преобразованием типов.
    EXPECT_EQ(m2(0, 0), 1.0);
    EXPECT_EQ(m2(1, 1), 4.0);
}

// Тест: Преобразование типов через конструктор
TEST(MatrixTests, TypeConversionConstructor) {
    linalg::Matrix<int> m1 = {{1, 2}, {3, 4}};
    linalg::Matrix<double> m2(m1); // Конструктор с преобразованием типов.
    EXPECT_EQ(m2(0, 0), 1.0);
    EXPECT_EQ(m2(1, 1), 4.0);
}

// Тест: Арифметические операции
TEST(MatrixTests, ArithmeticOperations) {
    linalg::Matrix<int> m1 = {{1, 2}, {3, 4}};
    linalg::Matrix<int> m2 = {{4, 3}, {2, 1}};

    linalg::Matrix<int> sum = m1 + m2;
    EXPECT_EQ(sum(0, 0), 5);
    EXPECT_EQ(sum(1, 1), 5);

    linalg::Matrix<int> diff = m1 - m2;
    EXPECT_EQ(diff(0, 0), -3);
    EXPECT_EQ(diff(1, 1), 3);

    linalg::Matrix<int> prod = m1 * m2;
    EXPECT_EQ(prod(0, 0), 8);
    EXPECT_EQ(prod(1, 1), 13);
}

// Тест: Операторы сравнения
TEST(MatrixTests, ComparisonOperators) {
    linalg::Matrix<int> m1 = {{1, 2}, {3, 4}};
    linalg::Matrix<int> m2 = {{1, 2}, {3, 4}};
    linalg::Matrix<int> m3 = {{1, 0}, {0, 4}};

    EXPECT_TRUE(m1 == m2);
    EXPECT_FALSE(m1 == m3);
    EXPECT_TRUE(m1 != m3);
}

// Тест: Резервирование памяти
TEST(MatrixTests, CapacityAndReserve) {
    linalg::Matrix<int> m(2, 3);
    EXPECT_GE(m.capacity(), 6);

    m.reserve(20);
    EXPECT_GE(m.capacity(), 20);

    m.shrink_to_fit();
    EXPECT_EQ(m.capacity(), 6); // После shrink_to_fit вместимость равна текущему размеру.
}

// Тест: Очистка матрицы
TEST(MatrixTests, ClearMatrix) {
    linalg::Matrix<int> m(2, 3);
    m.clear();
    EXPECT_EQ(m.rows(), 0);
    EXPECT_EQ(m.columns(), 0);
    EXPECT_EQ(m.size(), 0);
    EXPECT_GE(m.capacity(), 6); // Вместимость остаётся неизменной.
}

// Тест: Методы begin и end
TEST(MatrixTests, Iterators) {
    linalg::Matrix<int> m = {{1, 2, 3}, {4, 5, 6}};
    std::vector<int> elements(m.begin(), m.end());
    std::vector<int> expected = {1, 2, 3, 4, 5, 6};
    EXPECT_EQ(elements, expected);
}

// Тест: Преобразование типов при арифметических операциях
TEST(MatrixTests, ArithmeticWithTypeConversion) {
    linalg::Matrix<int> m1 = {{1, 2}, {3, 4}};
    linalg::Matrix<double> m2 = {{4.5, 3.5}, {2.5, 1.5}};

    linalg::Matrix<double> sum = m1 + m2;
    EXPECT_DOUBLE_EQ(sum(0, 0), 5.5);
    EXPECT_DOUBLE_EQ(sum(1, 1), 5.5);

    linalg::Matrix<double> prod = m2 * 2;
    EXPECT_DOUBLE_EQ(prod(0, 0), 9.0);
    EXPECT_DOUBLE_EQ(prod(1, 1), 3.0);
}
